/* valacodegen.h generated by valac, the Vala compiler, do not modify */


#ifndef __VALACODEGEN_H__
#define __VALACODEGEN_H__

#include <glib.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <valagee.h>
#include <glib-object.h>

G_BEGIN_DECLS

gchar* vala_get_ccode_name (ValaCodeNode* node);
gchar* vala_get_ccode_const_name (ValaCodeNode* node);
gchar* vala_get_ccode_type_name (ValaInterface* iface);
gchar* vala_get_ccode_lower_case_name (ValaCodeNode* node, const gchar* infix);
gchar* vala_get_ccode_upper_case_name (ValaSymbol* sym, const gchar* infix);
gchar* vala_get_ccode_header_filenames (ValaSymbol* sym);
gchar* vala_get_ccode_prefix (ValaSymbol* sym);
gchar* vala_get_ccode_lower_case_prefix (ValaSymbol* sym);
gchar* vala_get_ccode_lower_case_suffix (ValaSymbol* sym);
gchar* vala_get_ccode_ref_function (ValaTypeSymbol* sym);
gchar* vala_get_ccode_quark_name (ValaErrorDomain* edomain);
gboolean vala_is_reference_counting (ValaTypeSymbol* sym);
gboolean vala_get_ccode_ref_function_void (ValaClass* cl);
gboolean vala_get_ccode_free_function_address_of (ValaClass* cl);
gchar* vala_get_ccode_unref_function (ValaObjectTypeSymbol* sym);
gchar* vala_get_ccode_ref_sink_function (ValaObjectTypeSymbol* sym);
gchar* vala_get_ccode_copy_function (ValaTypeSymbol* sym);
gchar* vala_get_ccode_destroy_function (ValaTypeSymbol* sym);
gchar* vala_get_ccode_dup_function (ValaTypeSymbol* sym);
gchar* vala_get_ccode_free_function (ValaTypeSymbol* sym);
gboolean vala_get_ccode_is_gboxed (ValaTypeSymbol* sym);
gboolean vala_get_ccode_finish_instance (ValaCodeNode* node);
gchar* vala_get_ccode_type_id (ValaCodeNode* node);
gchar* vala_get_ccode_marshaller_type_name (ValaCodeNode* node);
gchar* vala_get_ccode_get_value_function (ValaCodeNode* sym);
gchar* vala_get_ccode_set_value_function (ValaCodeNode* sym);
gchar* vala_get_ccode_take_value_function (ValaCodeNode* sym);
gchar* vala_get_ccode_param_spec_function (ValaCodeNode* sym);
gchar* vala_get_ccode_type_check_function (ValaTypeSymbol* sym);
gchar* vala_get_ccode_default_value (ValaTypeSymbol* sym);
gboolean vala_get_ccode_has_copy_function (ValaStruct* st);
gboolean vala_get_ccode_has_destroy_function (ValaStruct* st);
gdouble vala_get_ccode_instance_pos (ValaCodeNode* node);
gboolean vala_get_ccode_array_length (ValaCodeNode* node);
gchar* vala_get_ccode_array_length_type (ValaCodeNode* node);
gboolean vala_get_ccode_array_null_terminated (ValaCodeNode* node);
gchar* vala_get_ccode_array_length_name (ValaCodeNode* node);
gchar* vala_get_ccode_array_length_expr (ValaCodeNode* node);
gdouble vala_get_ccode_array_length_pos (ValaCodeNode* node);
gdouble vala_get_ccode_delegate_target_pos (ValaCodeNode* node);
gdouble vala_get_ccode_destroy_notify_pos (ValaCodeNode* node);
gboolean vala_get_ccode_delegate_target (ValaCodeNode* node);
gchar* vala_get_ccode_delegate_target_name (ValaVariable* variable);
gdouble vala_get_ccode_pos (ValaParameter* param);
gchar* vala_get_ccode_type (ValaCodeNode* node);
gboolean vala_get_ccode_simple_generics (ValaMethod* m);
gchar* vala_get_ccode_real_name (ValaSymbol* sym);
gchar* vala_get_ccode_constructv_name (ValaCreationMethod* m);
gchar* vala_get_ccode_vfunc_name (ValaMethod* m);
gchar* vala_get_ccode_finish_name (ValaMethod* m);
gchar* vala_get_ccode_finish_vfunc_name (ValaMethod* m);
gchar* vala_get_ccode_finish_real_name (ValaMethod* m);
gboolean vala_get_ccode_no_accessor_method (ValaProperty* p);
gboolean vala_get_ccode_concrete_accessor (ValaProperty* p);
gboolean vala_get_ccode_has_type_id (ValaTypeSymbol* sym);
gboolean vala_get_ccode_has_new_function (ValaMethod* m);
gboolean vala_get_ccode_has_generic_type_parameter (ValaMethod* m);
gdouble vala_get_ccode_generic_type_pos (ValaMethod* m);
gchar* vala_get_ccode_sentinel (ValaMethod* m);

G_END_DECLS

#endif
